///////////////////////////////////////////////////////
////// Marin Radu
////// Grupa 334 CA
////// Proiect PM - I-volution
///////////////////////////////////////////////////////
///////////////////////////////////////////////////////
////// Include-uri
#define F_CPU 16000000
#include <avr\io.h>
#include <util\delay.h>
#include <stdio.h>
#include <stdint.h>
//////////////////////////////////////////////////////

//////////////////////////////////////////////////////
////// Tipuri de miscare
#define FORWARD 100		// fata
#define BACKWARD 101	// spate
#define LEFT 102		// stanga
#define RIGHT 103		// dreapta
#define STANDBY 104		// niciuna
///////////////////////////////////////////////////////

///////////////////////////////////////////////////////
/////// Masti pentru enable la motoare
#define MOTOR1 _BV(PB0)	// Trunchi dreapta fata
#define MOTOR2 _BV(PB1)	// Incheietura dreapta fata
#define MOTOR3 _BV(PB2)	// Incheietura dreapta spate
#define MOTOR4 _BV(PB3)	// Trunchi dreapta spate
#define MOTOR5 _BV(PB4)	// Trunchi stanga fata
#define MOTOR6 _BV(PB5)	// Incheietura stanga fata
#define MOTOR7 _BV(PB6)	// Incheietura stanga spate
#define MOTOR8 _BV(PB7)	// Trunchi stanga spate

#define TORSO (MOTOR1 | MOTOR4 | MOTOR5 | MOTOR8)	// trunchiul
///////////////////////////////////////////////////////

///////////////////////////////////////////////////////
////// Valori pentru PWM
#define MAX 2000
#define MED 1500
#define MIN 1000
#define STEP 500
///////////////////////////////////////////////////////

///////////////////////////////////////////////////////
////// Variabile globale
uint8_t movement;			// tipul de miscare
int offset_left = 0;		// offset pe stanga
int offset_right = 0;		// offset pe dreapta
///////////////////////////////////////////////////////

///////////////////////////////////////////////////////
////// Initializare TImer 1 pentru PWM
void init_timer(void)
{ 
	// activez ambele canale
    TCCR1A = _BV(COM1A1) | _BV(COM1B1);      
	// aleg Phase and Frequency Correct Mode + Prescaler 1/8
    TCCR1B = _BV(WGM13) | _BV(CS11);
	TCNT1H = 0x00;
	TCNT1L = 0x00;
	// initializez valorile
    ICR1 = 20000;	// frecventa de 50 Hz
	// activez pinii ca output pentru PWM
    DDRD = _BV(PD4) | _BV(PD5);	// 4 e cananul B, 5 e canalul A       
}

///////////////////////////////////////////////////////
////// comanda PWM pe canalul A
void PWM_channelA(uint8_t motor_mask, int value) {
	PORTB = motor_mask;		// dau enable motorului (sau motoarelor)
	OCR1A = value;			// stabilesc noua pozitie
	_delay_ms(1000);		// astept 20 ms (pentru PWM) + durata stabilirii mecanice a noii pozitii a servomotorului
}

///////////////////////////////////////////////////////
////// comanda PWM pe canalul B
void PWM_channelB(uint8_t motor_mask, int value) {
	PORTB = motor_mask;		// dau enable motorului (sau motoarelor)
	OCR1B = value;			// stabilesc noua pozitie
	_delay_ms(1000);		// astept 20 ms (pentru PWM) + durata stabilirii mecanice a noii pozitii a servomotorului
}

///////////////////////////////////////////////////////
////// miscare inainte
void move_forward() {
	if(offset_left > 0) {	// daca picioarele de pe partea stanga au ramas in spate
		// ele se vor misca primele
		// acum incep cu stangul din spate => offset pe dreapta
		offset_left = 0;
		offset_right = STEP;	// marchez ca partea dreapta va ramane in spate
		
		// picior stanga fata
		PWM_channelA(MOTOR6, MAX);	// ridic
		PWM_channelA(MOTOR5, MIN);	// deplasez
		PWM_channelA(MOTOR6, MED);	// cobor

		// picior stanga spate
		PWM_channelA(MOTOR7, MAX);	// ridic
		PWM_channelA(MOTOR8, MIN);	// deplasez
		PWM_channelA(MOTOR7, MED);	// cobor

		// miscare trunchi
		PORTB = TORSO;
		OCR1A = MED;
		OCR1B = MIN;
	} else {	// altfel se misca picioarele de pe partea dreapta
		// plec cu dreptul => offset pe stanga
		offset_right = 0;
		offset_left = STEP;	// marchez ca partea stanga va ramane in spate

		// picior dreapta fata
		PWM_channelB(MOTOR2, MIN);	// ridic
		PWM_channelB(MOTOR1, MAX);	// deplasez
		PWM_channelB(MOTOR2, MED);	// cobor
		
		// picior dreapta spate
		PWM_channelB(MOTOR3, MIN);	// ridic
		PWM_channelB(MOTOR4, MAX);	// deplasez
		PWM_channelB(MOTOR3, MED);	// cobor

		// miscare trunchi
		PORTB = TORSO;
		OCR1B = MED;
		OCR1A = MAX;
	}
}

///////////////////////////////////////////////////////
////// miscarea in spate
void move_backward() {
	if(offset_left > 0) {	// daca partea stanga e in spate incep cu ea
		// acum incep cu stangul din spate => offset pe dreapta
		offset_left = 0;
		offset_right = STEP;	// marchez ca cea dreapta va ramane in spate
		
		// picior stanga fata
		PWM_channelA(MOTOR6, MAX);	// ridic
		PWM_channelA(MOTOR5, MIN);	// deplasez
		PWM_channelA(MOTOR6, MED);	// cobor

		// picior stanga spate
		PWM_channelA(MOTOR7, MAX);	// ridic
		PWM_channelA(MOTOR8, MIN);	// deplasez
		PWM_channelA(MOTOR7, MED);	// cobor

		// miscare trunchi
		PORTB = TORSO;
		OCR1A = MED;
		OCR1B = MIN;
	} else {	// altfel incep cu cea dreapta
		// plec cu dreptul => offset pe stanga
		offset_right = 0;
		offset_left = STEP;

		// picior dreapta fata
		PWM_channelB(MOTOR2, MIN);	// ridic
		PWM_channelB(MOTOR1, MAX);	// deplasez
		PWM_channelB(MOTOR2, MED);	// cobor
		
		// picior dreapta spate
		PWM_channelB(MOTOR3, MIN);	// ridic
		PWM_channelB(MOTOR4, MAX);	//deplasez
		PWM_channelB(MOTOR3, MED);	// cobor

		// miscare trunchi
		PORTB = TORSO;
		OCR1B = MED;
		OCR1A = MAX;
	}
}

///////////////////////////////////////////////////////
////// miscare stanga
void move_left() {
	
	if(offset_left > 0) {
		offset_left = 0;	// daca partea stanga e in spate nu trebuie sa o mai deplaez
	} else {	// altfel
		// picior stanga fata
		PWM_channelA(MOTOR6, MAX);	// ridic
		PWM_channelA(MOTOR5, MIN);	// deplasez
		PWM_channelA(MOTOR6, MED);	// cobor

		// picior stanga spate
		PWM_channelA(MOTOR7, MAX);	// ridic
		PWM_channelA(MOTOR8, MIN);	// deplasez
		PWM_channelA(MOTOR7, MED);	// cobor
	}
	
	// mut si partea dreapta
	offset_right = 0;	// dupa miscarea trunchiului toate membrele vor fi perpendiculare pe ele

	// picior dreapta fata
	PWM_channelB(MOTOR2, MIN);	// ridic
	PWM_channelB(MOTOR1, MAX);	// deplaez
	PWM_channelB(MOTOR2, MED);	// cobor
	
	// picior dreapta spate
	PWM_channelB(MOTOR3, MIN);	// ridic
	PWM_channelB(MOTOR4, MAX);	// deplasez
	PWM_channelB(MOTOR3, MED);	// cobor

	// miscare trunchi
	PORTB = TORSO;
	OCR1B = MED;
	OCR1A = MED;
}

///////////////////////////////////////////////////////
////// miscare dreapta
void move_right() {
	if(offset_right > 0) {
		offset_right = 0;	// daca membrele de partea dreapta sunt deja in spate nu trebuie sa le mai deplasez
	} else {	// altfel
		// picior dreapta fata
		PWM_channelB(MOTOR2, MIN);	// ridic
		PWM_channelB(MOTOR1, MAX);	// deplasez
		PWM_channelB(MOTOR2, MED);	// cobor
		
		// picior dreapta spate
		PWM_channelB(MOTOR3, MIN);	// ridic
		PWM_channelB(MOTOR4, MAX);	// deplasez
		PWM_channelB(MOTOR3, MED);	// cobor
	}

	offset_left = 0;	// dupa miscarea trunchiului toate membrele vor fi perpendiculare pe ele

	// picior stanga fata
	PWM_channelA(MOTOR6, MAX);	// ridic
	PWM_channelA(MOTOR5, MIN);	// deplasez
	PWM_channelA(MOTOR6, MED);	// cobor

	// picior stanga spate
	PWM_channelA(MOTOR7, MAX);	// ridic
	PWM_channelA(MOTOR8, MIN);	// deplasez
	PWM_channelA(MOTOR7, MED);	// cobor

	// miscare trunchi
	PORTB = TORSO;
	OCR1B = MED;
	OCR1A = MED;
}

///////////////////////////////////////////////////////
////// readucerea picioarelor la pozitia initiala (perpendiculara pe trunchi)
void stop() {
	if(offset_right > 0) {
		// picior dreapta fata
		PWM_channelB(MOTOR2, MIN);	// ridic
		PWM_channelB(MOTOR1, MED);	// deplasez
		PWM_channelB(MOTOR2, MED);	// cobor
		
		// picior dreapta spate
		PWM_channelB(MOTOR3, MIN);	// ridic
		PWM_channelB(MOTOR4, MED);	// deplasez
		PWM_channelB(MOTOR3, MED);	// cobor
	}

	if(offset_left > 0) {
		// picior stanga fata
		PWM_channelA(MOTOR6, MAX);	// ridic
		PWM_channelA(MOTOR5, MED);	// deplasez
		PWM_channelA(MOTOR6, MED);	// cobor

		// picior stanga spate
		PWM_channelA(MOTOR7, MAX);	// ridic
		PWM_channelA(MOTOR8, MED);	// deplasez
		PWM_channelA(MOTOR7, MED);	// cobor
	}
}
///////////////////////////////////////////////////////

///////////////////////////////////////////////////////
int main(void)
{	
	// initial se deplaseaza inainte
	movement = FORWARD;
	offset_left = offset_right = 0;	// picioarele perpendiculare pe trunchi
	
	// activez butonul de la PD6
	DDRD &= ~_BV(PD6);
	PORTD |= _BV(PD6);
	// aprind ledul de la PD&
	DDRD |= _BV(PD7);
	PORTD |= _BV(PD7);
	
	// astept apasarea butonului pentru a stinge ledul si a porni algorimtul
	while((PIND & (1 << PD6)) == (1 << PD6));
	
    init_timer();	// initializez timer-ul 1 pe PWM
	_delay_ms(1000);	// astept un pic sa se stabilizeze
	DDRB = 0xFF;	// portul de enable e pus pe output LOW
	
	// la nesfarsit
	while(1) {
		// in functie de miscare apelez functia care o realizeaza
		switch(movement) {
			case FORWARD:
				move_forward();
				break;
			case BACKWARD:
				move_backward();
				break;
			case LEFT:
				move_left();
				break;
			case RIGHT:
				move_right();
				break;
			default:
				stop();
		}
	}

	return 0;
}
///////////////////////////////////////////////////////
